/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#define sunIllumMult 1.0    //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define moonIllumMult 1.0   //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

#define airRayleighMult 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define airMieMult 1.0      //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define airOzoneMult 1.0    //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

const float planetRad   = 6371e3;
const float atmosDepth  = 110e3;
const float atmosRad    = planetRad + atmosDepth;

const float ozonePeakAlt = 38e3;

const float airMieG     = 0.74;

//const vec3 airRayleighCoeff = vec3(9.11388125e-6, 1.32236310e-5, 3.03829375e-5) * airRayleighMult;
const vec3 airRayleighCoeff = vec3(9.21388125e-6, 1.22236310e-5, 3.03829375e-5) * airRayleighMult;
const vec3 airMieCoeff      = vec3(9e-6) * airMieMult;
const vec3 airOzoneCoeff    = vec3(3.58768136e-7, 3.14953105e-7, 5.43182681e-8) * 2.2 * airOzoneMult;

const vec2 scaleHeight      = vec2(8.5e3, 1.2e3);
const vec2 rScaleHeight     = 1.0 / scaleHeight;
const vec2 planetScale      = planetRad * rScaleHeight;

const vec3 sunIllum         = vec3(1.0, 0.97257505, 0.961118) * 60.0 * sunIllumMult;
//const vec3 sunIllum         = vec3(1.0, 0.969, 0.956) * 60.0 * sunIllumMult;
const vec3 moonIllum        = vec3(0.6, 0.67, 1.0) * 0.04 * moonIllumMult;

const mat2x3 airScatterMat  = mat2x3(airRayleighCoeff, airMieCoeff);
const mat3x3 airExtinctMat  = mat3x3(airRayleighCoeff, airMieCoeff * 1.1, airOzoneCoeff);

const vec3 fogMistCoeff     = vec3(1e-2);
const mat3x3 fogScatterMat  = mat3x3(airRayleighCoeff, airMieCoeff, fogMistCoeff);
const mat3x3 fogExtinctMat  = mat3x3(airRayleighCoeff, airMieCoeff * 1.1, fogMistCoeff);

const vec2 fogFalloffScale  = 1.0 / vec2(5e1, 25e1);
const vec2 fogAirScale      = vec2(100.0, 1e2);