/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

float planetCurveAltitude(vec3 x) {
    //return length(x);
    return length(x + vec3(0.0, planetRad, 0.0)) - planetRad;
}

uniform vec3 airDensityCoeff;

float rsi0(vec3 pos, vec3 dir, float r) {
    float b     = dot(pos, dir);
    float det   = sqr(b) - dot(pos, pos) + sqr(r);

    if (det < 0.0) return -1.0;

        det     = sqrt(det);

    float t1    = -b - det;
    float t2    = -b + det;

    return t1 >= 0.0 ? t1 : t2;
}
vec2 rsi(vec3 pos, vec3 dir, float r) {
    float b     = dot(pos, dir);
    float det   = sqr(b) - dot(pos, pos) + sqr(r);

    if (det < 0.0) return vec2(-1.0);

        det     = sqrt(det);

    return vec2(-b) + vec2(-det, det);
}

float airRSI(vec3 pos, vec3 dir) {
    vec2 planet = rsi(pos, dir, planetRad);
    bool isPlanet = planet.y >= 0.0;

    vec2 aDist  = rsi(pos, dir, atmosRad);
    vec2 pDist  = rsi(pos, dir, planetRad - 2.6e3);

    vec2 dist   = vec2(0.0);
        dist.x  = isPlanet && pDist.x < 0.0 ? pDist.y : max(aDist.x, 0.0);
        dist.y  = isPlanet && pDist.x > 0.0 ? pDist.x : aDist.y;

    return abs(dist.y - dist.x);
}

vec3 airRho(float elevation) {
    vec2 rm     = expf(-elevation * rScaleHeight + planetScale);
        //rm.x   *= 1.0 + wetness * 0.5;
        //rm.y   *= 1.0 + wetness * 40.0;

    float ozone = expf(-max(0.0, (ozonePeakAlt - elevation) - planetRad) * rcp(8.5e3)) * expf(-max(0.0, (elevation - ozonePeakAlt) - planetRad) * rcp(18.5e3));

    return vec3(rm, ozone) * airDensityCoeff;
}

vec3 airmass(vec3 pos, vec3 dir, float dist, const uint steps) {
    float rlength   = dist * rcp(float(steps));
    vec3 rstep      = dir * rlength;
        pos        += rstep * 0.5;

    vec3 rho        = vec3(0.0);

    for (uint i = 0; i < steps; ++i, pos += rstep) {
        if (rho.y > 1e35) break;
        rho    += airRho(planetCurveAltitude(pos));
    }
    return rho * rlength;
}
vec3 airmass(vec3 pos, vec3 dir, const uint steps) {
    float dist  = rsi0(pos, dir, atmosRad);

    return airmass(pos, dir, dist, steps);
}

vec3 airTransmittance(vec3 pos, vec3 dir, const uint steps) {
    return max(expf(-airExtinctMat * airmass(pos, dir, steps)), 0.0);
}
vec3 airTransmittance(vec3 pos, vec3 dir) {
    return airTransmittance(pos, dir, 4);
}
vec3 airDensity(vec3 dir, vec3 pos, const uint steps) {
    float dist  = airRSI(pos, dir);
    vec3 rho    = airRho(planetCurveAltitude(pos)) * dist;
    vec3 total  = rho + airmass(pos, dir, dist, steps);

    return total;
}
vec3 airDensity(vec3 dir, vec3 pos) {
    return airDensity(dir, pos, 4);
}