/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


const vec3 light_coeff  = vec3(0.3, 0.55, 1.0);
const vec3 zenith_coeff = vec3(0.10, 0.23, 1.0);

const vec3 sun_illum    = vec3(1.0, 0.94, 0.92) * 5.0;
const vec3 moon_illum   = vec3(0.8, 0.9, 1.0) * 3e-2;

const float density_coeff = 0.55;
const float horizon_offset = -0.04;

float atmos_density(float x) {
    return density_coeff * rcp(pow(max(x - horizon_offset, 0.35e-3), 0.95));
}
vec3 atmos_absorbtion(vec3 x, float y){
	vec3 absorption = x * -y;
	     absorption = exp(absorption) * 2.0;
    
    return absorption;
}

float atmos_rayleigh(vec3 dir, vec3 lightvec){
	return 1.0 + sqr(1.0 - clamp(distance(dir, lightvec), 0.0, 1.0)) * pi * 0.5;
}

vec3 atmos_light(vec3 lightvec) {
    vec3 magic_ozone = light_coeff * mix(vec3(1.0, 1.15, 1.0), vec3(1.0), sstep(lightvec, 0.0, 0.2));
    
    return atmos_absorbtion(magic_ozone, atmos_density(lightvec.y));
}

vec3 atmos_approx(vec3 dir, vec3 sunvec, vec3 moonvec) {
    float vDotS = dot(sunvec, dir);
    float vDotM = dot(moonvec, dir);

    mat2x3 phase    = mat2x3(rayleighPhase(vDotS), mieCS(vDotS, 0.74), mieHG(vDotS, 0.65),
                        rayleighPhase(vDotM), mieCS(vDotM, 0.74), mieHG(vDotM, 0.65));
                
    float sun_mult  = sqrt(saturate(length(max(sunvec.y - horizon_offset, 0.0)))) * 0.9;
    float moon_mult = sqrt(saturate(length(max(moonvec.y - horizon_offset, 0.0)))) * 0.9;

    vec3 magic_ozone = zenith_coeff * mix(vec3(1.0, 1.1, 1.0), vec3(1.0), sstep(max(sunvec.y, moonvec.y), 0.0, 0.2));

    float density   = atmos_density(dir.y);
    vec3 absorption = atmos_absorbtion(magic_ozone, density);

    vec3 sunlight   = atmos_light(sunvec) * sun_illum;
    vec3 moonlight  = atmos_light(moonvec) * moon_illum;

    float sun_ms    = phase[0].x * sstep(sunvec.y, horizon_offset, horizon_offset + 0.2) * 1.5 + phase[0].z * 0.5 + 0.1;
        sun_ms     *= 0.5 + sstep(sunvec.y, horizon_offset, horizon_offset + 0.4) * 0.5;

    float moon_ms   = phase[1].x * sstep(moonvec.y, horizon_offset, horizon_offset + 0.2) * 1.5 + phase[1].z;
        moon_ms    *= 0.5 + sstep(moonvec.y, horizon_offset, horizon_offset + 0.4) * 0.5;

    float sun_visibility = sstep(sunvec.y, -0.14, horizon_offset);
        phase[0].y *= sun_visibility;

    float moon_visibility = sstep(sunvec.y, -0.14, horizon_offset);
        phase[1].y *= moon_visibility;
    
    //float sun_rmult  = atmos_rayleigh(dir, sunvec);
    vec3 sun_scatter = zenith_coeff * density;
        sun_scatter  = mix(sun_scatter * absorption, mix(1.0 - exp2(-0.5 * sun_scatter), 0.5 * magic_ozone / (1.0 + magic_ozone), 1.0 - exp2(-0.25 * density)), sun_mult);
        sun_scatter *= sunlight * 0.5 + (0.5 * length(sunlight));
        sun_scatter += (1.0 - exp(-density * magic_ozone)) * sun_ms * sunlight;
        sun_scatter += sunlight * (phase[0].y * rcp(pi));

    //float moon_rmult  = atmos_rayleigh(dir, moonvec);
    vec3 moon_scatter = zenith_coeff * density;
        moon_scatter  = mix(moon_scatter * absorption, mix(1.0 - exp2(-0.5 * moon_scatter), 0.5 * magic_ozone / (1.0 + magic_ozone), 1.0 - exp2(-0.25 * density)), moon_mult);
        moon_scatter *= moonlight * 0.5 + (0.5 * length(moonlight));
        moon_scatter += (1.0 - exp(-density * magic_ozone)) * moon_ms * moonlight;
        moon_scatter += moonlight * (phase[1].y * rcp(pi));
        moon_scatter  = mix(moon_scatter, dot(moon_scatter, vec3(1.0/3.0)) * vec3(0.2, 0.55, 1.0), 0.8);

    vec3 result     = (sun_scatter) + (moon_scatter);

    return result * rcp(pi);
}