/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#define sunlight_luma 1.0
#define skylight_luma 1.0
#define blocklightIllum 1.0

#ifdef cloudPass
    const float eyeAltitude = 800.0;
#else
    uniform float eyeAltitude;
#endif

uniform vec2 viewSize;

uniform vec3 sunvec;
uniform vec3 moonvec;

uniform vec4 daytime;

uniform sampler2D gaux2;

#ifdef skyboxPass
    flat out vec3 airIlluminance;
#else
    flat out mat4x3 lightColor;
#endif

#ifdef skyboxPass
    #include "/lib/atmos/phase.glsl"
    #include "/lib/atmos/air/atmosphere.glsl"
#else
    #include "/lib/atmos/air/const.glsl"
    #include "/lib/atmos/air/rho.glsl"
    #include "/lib/atmos/project.glsl"
#endif

void make_colors() {
    #ifdef skyboxPass
        airIlluminance = atmosphericScatter(vec3(0.0, 1.0, 0.0), mat2x3(sunvec, moonvec), vec3(0.0));
        airIlluminance = mix(airIlluminance, vec3(airIlluminance.z), 0.33) * rcp(pi);
    #else
        vec3 airEyePos = vec3(0.0, planetRad + eyeAltitude, 0.0);

        lightColor[0]  = airTransmittance(airEyePos, sunvec, 6) * sunIllum;
        
        #ifdef fogPass
            lightColor[1]  = texture(gaux2, projectSky(vec3(0.0, 1.0, 0.0))).rgb;
        #else
            lightColor[1]  = texture(gaux2, projectSky(vec3(0.0, 1.0, 0.0))).rgb * sqrt2;
        #endif

        lightColor[2]  = airTransmittance(airEyePos, moonvec) * moonIllum;
        lightColor[3]  = blackbody(2000.0) * blocklightIllum * pi;

        //lightColor[3]  = vec3(0.4, 0.7, 1.0)*0.05;
    #endif
}