#version 400 compatibility

/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#include "/lib/head.glsl"
#include "/lib/util/encoders.glsl"

/*
stand in for sky pass
*/

in vec2 coord;

flat in mat2x3 lightColor;
flat in vec3 sky_color;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D depthtex1;

uniform sampler2D noisetex;

uniform int frameCounter;

uniform float aspectRatio;
uniform float far, near;

uniform mat4 gbufferProjection;

/* ------ includes ------ */
#define FUTIL_LINDEPTH
#define FUTIL_D3X3
#include "/lib/fUtil.glsl"

#include "/lib/frag/bluenoise.glsl"


/* ------ ambient occlusion and gi ------ */
#include "/lib/light/ao.glsl"

void main() {
    vec4 sceneColor   = stex(colortex0);
    float sceneDepth = stex(depthtex1).x;

    if (!landMask(sceneDepth)) {
        sceneColor.rgb = sky_color;
    }

    vec4 return3    = vec4(0.0);
    vec3 return6    = vec3(0.0);

    #ifdef lightmapSmoothingEnabled
    if (landMask(sceneDepth)) {
        vec4 tex1   = stex(colortex1);
        return3.xyz = vec3(decode2x8(tex1.z), sceneColor.a);
        return6     = decodeNormal(tex1.xy) * 0.5 + 0.5;
    }
    #endif

    #ifdef ambientOcclusionEnabled
        vec2 ao_coord   = (coord)*2.0;

        if (clamp(ao_coord, -0.003, 1.003) == ao_coord) {
            vec2 coord  = ao_coord;
            float d     = depthMax3x3(depthtex1, ao_coord, pixelSize*sqrt(2.0));

            if (landMask(d)) {
                float sceneDepth = stex(depthtex1).x;
                vec4 tex1       = stex(colortex1);

                vec3 viewpos    = screenToViewSpace(vec3(coord, sceneDepth));

                vec3 sceneNormal = decodeNormal(tex1.xy);
                vec3 viewnormal = normalize(mat3(gbufferModelView) * sceneNormal);

                float ao    = calculate_dbao(depthtex1, sceneDepth, coord, ditherBluenoise());
                //float ao    = calculate_ssao(depthtex1, viewpos, viewnormal, ditherBluenoise());

                return3.w   = ao;
            } else {
                return3.w   = 1.0;
            }
        }
    #endif

    /*DRAWBUFFERS:036*/
    gl_FragData[0]  = makeDrawbuffer(sceneColor);
    gl_FragData[1]  = clampDrawbuffer(return3);
    gl_FragData[2]  = clampDrawbuffer(return6);
}