#version 400 compatibility

/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#include "/lib/head.glsl"

#if (defined lightHqFilterEnabled && defined lightmapSmoothingEnabled)
/*DRAWBUFFERS:35*/
layout(location = 0) out vec4 lightmapData;
layout(location = 1) out vec4 skyboxData;
#else
/*DRAWBUFFERS:5*/
layout(location = 0) out vec4 skyboxData;
#endif

#define ATROUS_SIZE 32

#include "/lib/util/colorspace.glsl"

in vec2 coord;

flat in mat4x3 lightColor;

uniform sampler2D colortex3;
uniform sampler2D colortex5;
uniform sampler2D colortex6;

uniform sampler2D depthtex0;

uniform sampler2D noisetex;
uniform sampler3D depthtex1;
uniform sampler3D depthtex2;

//uniform sampler3D colortex2;

uniform int frameCounter;
uniform int worldTime;

uniform float eyeAltitude;
uniform float far, near;
uniform float frameTimeCounter;

uniform vec2 pixelSize;
uniform vec2 viewSize;
uniform vec2 taaOffset;
uniform vec2 skyCaptureResolution;

uniform vec3 cameraPosition;
uniform vec3 upvec, upvecView;
uniform vec3 sunvec, sunvecView;
uniform vec3 moonvec, moonvecView;
uniform vec3 cloudLightDir, cloudLightDirView;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;


/* ------ includes ------*/
#define FUTIL_LINDEPTH
#define FUTIL_D3X3
#include "/lib/fUtil.glsl"

#include "/lib/util/kernel.glsl"
#include "/lib/light/atrousLightmap.glsl"


#include "/lib/util/transforms.glsl"
#include "/lib/atmos/phase.glsl"
#include "/lib/frag/bluenoise.glsl"
#include "/lib/atmos/air/const.glsl"
#include "/lib/atmos/air/rho.glsl"
#include "/lib/atmos/project.glsl"
#include "/lib/frag/noise.glsl"
#include "/lib/util/bicubic.glsl"

/*
float bluenoiseLookup1D() {     //surprisingly this works less good with taa
    ivec2 xy    = ivec2(gl_FragCoord.xy) & 255;
    uint z      = frameCounter & 7;

    return texelFetch(colortex2, ivec3(xy, z), 0).x;
}*/

/* ------ functions ------ */
#define cloudVolumeSamples 50
#define cloudTransmittanceThreshold 0.05
#define cloudVolumeClip 6e4
#define cloudCumulusAlt 800.0
#define cloudCumulusDepth 2000.0
#define cloudCumulusScale 15e-5
#define cloudCumulusCoverageBias 0.0

const float cloudCumulusMaxY    = cloudCumulusAlt + cloudCumulusDepth;
const float cloudCumulusMidY    = cloudCumulusAlt + cloudCumulusDepth * 0.5;

float cloudTime     = frameTimeCounter * 0.005;

uniform vec3 volumeCloudData;

float mieCloud(float cosTheta, float g) {
    float sqrG  = sqr(g);
    float a     = (1.0 - sqrG) * rcp(2.0 + sqrG);
    float b     = (1.0 + sqr(cosTheta)) * rcp((-2.0 * (g * cosTheta)) + 1.0 + sqrG);

    return max((1.5 * (a * b)) + (g * cosTheta), 0.0) * rpi;
}
/*
float cloudPhase(float cosTheta, float g) {
    float x = mieHG(cosTheta, g * 0.45) + mieCloud(cosTheta, 0.89 * g);
    float y = mieCloud(cosTheta, -g * 0.25) * volumeCloudData.z;

    return mix(x, y, 0.38);
}
*/

float cloudPhase(float cosTheta, float g, vec3 gMult) {
    float x = mieHG(cosTheta, gMult.x * g);
    float y = mieCloud(cosTheta, -gMult.y * g) * volumeCloudData.z;
    float z = mieCloud(cosTheta, gMult.z * g);

    return mix(mix(x, y, 0.19), z, 0.19) + 0.02 * g;    //i assume this is more energy conserving than summing them
}
float cloudSkyPhase(float cosTheta, float g, vec3 gMult) {
    float x = mieHG(cosTheta, gMult.x * g);
    float y = mieCloud(cosTheta, -gMult.y * g) * volumeCloudData.z;
    //float z = mieCloud(cosTheta, gMult.z * g);

    return mix(x, y, 0.19);    //i assume this is more energy conserving than summing them
}

vec3 curl3D(vec3 pos) {
    return texture(depthtex1, fract(pos)).xyz;
}
vec3 noise3D(vec3 pos) {   //x-large scale worley, y-small scale worley, z-perlin-worley
    return texture(depthtex2, fract(pos)).xyz;
}

float cloudCumulusShape(vec3 pos, out float powderBias) {
    powderBias = 0.0;
    float altitude  = pos.y;
    float altRemapped = saturate((altitude - cloudCumulusAlt) * rcp(cloudCumulusDepth));

    float erodeLow  = 1.0 - sstep(altRemapped, 0.0, 0.22);
    float erodeHigh = sstep(altRemapped, 0.19, 1.0);
    float fadeLow   = sstep(altRemapped, 0.0, 0.2);
    float fadeHigh  = 1.0 - sstep(altRemapped, 0.6, 1.0);

    vec3 wind       = vec3(cloudTime, 0.0, cloudTime * 0.4);

        pos         = (pos * cloudCumulusScale) + wind;

    float coverageBias = 0.47 + cloudCumulusCoverageBias + volumeCloudData.x;

    float coverage  = noise2D(pos.xz * 0.24).z;
        coverage    = (coverage - coverageBias) * rcp(1.0 - saturate(coverageBias) * 0.999);

        coverage    = (coverage * fadeLow * fadeHigh) - erodeLow * 0.26 - erodeHigh * 0.7;

        coverage    = saturate(coverage * 1.05);

    if (coverage <= 1e-8) return 0.0;

    float wfade     = 1.0 - sstep(altRemapped, 0.0, 0.33) * 0.93;
    //float curlScale = wfade + (1.0 - sstep(altRemapped, 0.0, 0.2));

    float dfade     = 0.001 + sstep(altRemapped, 0.0, 0.2) * 0.15;
        dfade      += sstep(altRemapped, 0.1, 0.55) * 0.65;
        dfade      += sstep(altRemapped, 0.2, 0.65) * 0.9;      //can be increased in regular kappa with more samples
        dfade      += sstep(altRemapped, 0.4, 0.85) * 0.7;


    float shape     = coverage;
    float slope     = sqrt(1.0 - saturate(shape));

    if (slope <= 1e-12) return 0.0;

    float hardness  = sqrt(sstep(altRemapped, 0.32, 0.64));

    vec3 curl       = curl3D(pos * 4.0) * 2.0 - 1.0;
        pos        += (curl * 0.15 * wfade) + wind * 0.3;

    vec3 noise      = noise3D((pos * 2.8));

    float n         = noise.x + sqrt(noise.y) * 0.6;
        n          *= rcp(1.6);

        shape      -= (1.0 - n) * 0.37 * slope;
        shape       = thresholdStep(shape, 0.08 * hardness);

    if (shape <= 0.0) return 0.0;
        pos        += shape * 0.1;

        slope     = sqrt(1.0 - saturate(shape));

        shape      -= value3D(pos * 256.0) * (0.05 + 0.06 * hardness) * slope;

        powderBias  = max(1.0 - hardness * 0.85, 0.36 * rcp(volumeCloudData.y));

    return max(shape * dfade * volumeCloudData.y, 0.0);
}
float cloudCumulusShape(vec3 pos) {
    float t = 0.0;
    return cloudCumulusShape(pos, t);
}

float cloudVolumeLightOD(vec3 pos, const uint steps) {
    //float basestep = mix(150.0, 20.0, saturate(float(steps-1)/4.0));
    //float exponent = mix(2.5, 1.1, saturate(float(steps-8)/8.0));
    float basestep = 22.2;
    float exponent = 2.0;

    float stepsize  = basestep;
    float prevStep  = stepsize;

    float od = 0.0;

    for(uint i = 0; i < steps; ++i, pos += cloudLightDir * stepsize) {

        if(pos.y > cloudCumulusMaxY || pos.y < cloudCumulusAlt) continue;

            prevStep  = stepsize;
            stepsize *= exponent;
        
        float density = cloudCumulusShape(pos);
        if (density <= 0.0) continue;

            od += density * prevStep;
    }

    return od;
}
float cloudVolumeSkyOD(vec3 pos, const uint steps) {
    const vec3 dir = vec3(0.0, 1.0, 0.0);

    float stepsize = (cloudCumulusDepth / float(steps));
        stepsize  *= 1.0-linStep(pos.y, cloudCumulusDepth, cloudCumulusMaxY) * 0.9;

    float od = 0.0;

    for(uint i = 0; i < steps; ++i, pos += dir * stepsize) {

        if(pos.y > cloudCumulusMaxY || pos.y < cloudCumulusAlt) continue;
        
        float density = cloudCumulusShape(pos);
        if (density <= 0.0) continue;

            od += density * stepsize;
    }

    return od;
}
float cloudVolumeSkyOD(vec3 pos, const uint steps, float noise) {
    const vec3 dir = vec3(0.0, 1.0, 0.0);

    float stepsize = (cloudCumulusDepth / float(steps));
        stepsize  *= (1.0-linStep(pos.y, cloudCumulusDepth, cloudCumulusMaxY))*0.9+0.1;

        pos += dir * stepsize * noise;

    float od = 0.0;

    for(uint i = 0; i < steps; ++i, pos += dir * stepsize) {

        if(pos.y > cloudCumulusMaxY || pos.y < cloudCumulusAlt) continue;
        
        float density = cloudCumulusShape(pos);
        if (density <= 0.0) continue;

            od += density * stepsize;
    }

    return od;
}


/* ------ cirrus clouds ------ */
#define cloudCirrusClip 2e5
#define cloudCirrusAlt 8000.0
#define cloudCirrusDepth 3000.0
#define cloudCirrusScale 3e-5
#define cloudCirrusCoverageBias 0.0

const float cloudCirrusMaxY     = cloudCirrusAlt + cloudCirrusDepth;
const float cloudCirrusPlaneY   = cloudCirrusAlt + cloudCirrusDepth * 0.2;

float cloudCirrusShape(vec3 pos) {
    float altitude  = pos.y;
    float altRemapped = saturate((altitude - cloudCirrusAlt) * rcp(cloudCirrusDepth));

    float erodeLow  = 1.0 - sstep(altRemapped, 0.0, 0.19);
    float erodeHigh = sstep(altRemapped, 0.22, 1.0);
    float fadeLow   = sstep(altRemapped, 0.0, 0.2);
    float fadeHigh  = 1.0 - sstep(altRemapped, 0.6, 1.0);

    vec3 wind       = vec3(cloudTime, 0.0, -cloudTime * 0.2) * 0.5;

        pos        *= rcp(1.0 + distance(cameraPosition, pos) * rcp(cloudCirrusClip));

        pos.x      *= 0.6;

        pos         = (pos * cloudCirrusScale) + wind;

    float coverageBias = 0.42 + cloudCirrusCoverageBias;

    float coverage  = noise2D(pos.xz * 0.23 * vec2(0.7, 1.0)).z;
        coverage    = (coverage - coverageBias) * rcp(1.0 - saturate(coverageBias) * 0.999);

        coverage    = (coverage * fadeLow * fadeHigh) - erodeLow * 0.26 - erodeHigh * 0.7;

        coverage    = saturate(coverage * 1.05);

    if (coverage <= 1e-8) return 0.0;

    float wfade     = 1.0 - sstep(altRemapped, 0.0, 0.33) * 0.95;

    const float dfade     = 0.33;
        //dfade      += sstep(altRemapped, 0.2, 0.55) * 0.2;
        //dfade      += sstep(altRemapped, 0.2, 0.65) * 0.5;
        //dfade      += sstep(altRemapped, 0.4, 0.85) * 0.5;


    float shape     = coverage;
    float slope     = sqrt(1.0 - saturate(shape));

    if (slope <= 1e-12) return 0.0;

    vec3 curl       = curl3D(pos * 1.5 - vec3(0.5, 0.0, 0.5)) * 2.0 - 1.0;
        pos        += curl * 0.06 * vec3(1.0, 2.0, 1.0);

    vec3 noise      = noise3D(pos * 1.6);

    float n         = noise.z + sqrt(noise.y) * 0.6;
        n          *= rcp(1.6);

        shape      -= (1.0 - n) * 0.3 * slope;

    if (shape <= 0.0) return 0.0;
        pos        += shape * 0.05;

        slope     = sqrt(1.0 - saturate(shape));

        //pos.x *= 2.0;

        shape      -= value3D(pos * 224.0) * 0.04 * slope;

        shape   = sqr(max(0.0, shape));

    return max(shape, 0.0);
}
float cloudCirrusLightOD(vec3 pos, const uint steps, vec3 dir) {
    float stepsize = (cloudCirrusDepth / float(steps));
        stepsize  *= (1.0-linStep(pos.y, cloudCirrusDepth, cloudCirrusMaxY))*0.9+0.1;

    float od = 0.0;

    for(uint i = 0; i < steps; ++i, pos += dir * stepsize) {

        if(pos.y > cloudCirrusMaxY || pos.y < cloudCirrusAlt) continue;
        
        float density = cloudCirrusShape(pos);
        if (density <= 0.0) continue;

            od += density * stepsize;
    }

    return od;
}

float ditherGradNoise(){
    return fract(52.9829189*fract(0.06711056*gl_FragCoord.x + 0.00583715*gl_FragCoord.y));
} 
float ditherGradNoiseTemporal(){
    return fract(52.9829189*fract(0.06711056*gl_FragCoord.x + 0.00583715*gl_FragCoord.y + 0.00623715 * (frameCounter * 0.31)));
}

/* ------ cloud system function ------ */

vec4 cloudSystem(vec3 worldDir, float vDotL, float dither, vec3 skyColor) {
    vec3 totalScattering    = vec3(0.0);
    float totalTransmittance = 1.0;

    vec3 sunlight       = (worldTime>23000 || worldTime<12900) ? lightColor[0] : lightColor[2];
    vec3 skylight       = lightColor[1] * rcp(2.0);

    float pFade         = saturate(mieHG(vDotL, 0.65));

    // --- volumetric clouds --- //
    #ifdef cloudVolumeEnabled
    float within    = sstep(eyeAltitude, cloudCumulusAlt - 75.0, cloudCumulusAlt) * (1.0 - sstep(eyeAltitude, cloudCumulusMaxY, cloudCumulusMaxY + 75.0));
    bool isBelowVol = eyeAltitude < cloudCumulusMidY;
    bool visibleVol = worldDir.y > 0.0 && isBelowVol || worldDir.y < 0.0 && !isBelowVol;

    float lightNoise = ditherGradNoiseTemporal();

    if (visibleVol || within > 0.0) {

        vec3 bottom     = worldDir * ((cloudCumulusAlt - eyeAltitude) * rcp(worldDir.y));
        vec3 top        = worldDir * ((cloudCumulusMaxY - eyeAltitude) * rcp(worldDir.y));

        float airDist   = length(worldDir.y < 0.0 ? bottom : top);
            airDist     = min(airDist, cloudVolumeClip);

        vec3 airmass    = airmass(vec3(0.0, planetRad + eyeAltitude, 0.0), worldDir, airDist, 3) * rcp(40000.0);

        if (worldDir.y < 0.0 && isBelowVol || worldDir.y > 0.0 && !isBelowVol) {
            bottom      = vec3(0.0);
            top         = vec3(0.0);
        }

        vec3 start      = isBelowVol ? bottom : top;
        vec3 end        = isBelowVol ? top : bottom;
            start       = mix(start, gbufferModelViewInverse[3].xyz, within);
            end         = mix(end, worldDir * 6e4, within);

        const float baseLength  = cloudCumulusDepth / cloudVolumeSamples;
        float stepLength    = length(end - start) * rcp(float(cloudVolumeSamples));
        float stepCoeff     = stepLength * rcp(baseLength);
            stepCoeff       = 0.45 + clamp(stepCoeff - 1.1, 0.0, 3.5) * 0.5;
            stepCoeff       = mix(stepCoeff, 10.0, sqr(within));

        uint steps          = uint(cloudVolumeSamples * stepCoeff);

        vec3 rStep          = (end - start) * rcp(float(steps));
        vec3 rPos           = rStep * dither + start + cameraPosition;
        float rLength       = length(rStep);

        vec3 scattering     = vec3(0.0);
        float transmittance = 1.0;

        vec3 bouncelight    = vec3(0.6, 1.0, 0.8) * sunlight * rcp(pi * 12.0);

        const float sigmaA  = 1.0;
        const float sigmaT  = 0.09;

        for (uint i = 0; i < steps; ++i, rPos += rStep) {
            if (transmittance < cloudTransmittanceThreshold) break;
            if (rPos.y < cloudCumulusAlt || rPos.y > cloudCumulusMaxY) continue;

            float dist  = distance(rPos, cameraPosition);
            if (dist > cloudVolumeClip) continue;

            float powderBias;
            float density = cloudCumulusShape(rPos, powderBias);
            if (density <= 0.0) continue;

            float extinction    = density * sigmaT;
            float stepT         = exp(-extinction * rLength);
            float integral      = (1.0 - stepT) * rcp(sigmaT);

            float airmassMult = 1.0 + sstep(dist, cloudVolumeClip * 0.9, cloudVolumeClip); 
            vec3 atmosFade = expf(-max(airScatterMat * airmass.xy * (dist * airmassMult), 0.0));

            if (max3(atmosFade) < 1e-4) {
                scattering     += (skyColor * sigmaT) * (integral * transmittance);
                transmittance  *= stepT;

                continue;
            }

            vec3 stepScatter    = vec3(0.0);

            float lightOD       = cloudVolumeLightOD(rPos, 6);
            
            #ifdef cloudVolumeSkylightDither
                float skyOD         = cloudVolumeSkyOD(rPos, 4, lightNoise);
            #else
                float skyOD         = cloudVolumeSkyOD(rPos, 4);
            #endif
            
            float bounceOD      = cube(1.0 - linStep(rPos.y, cloudCumulusAlt, cloudCumulusAlt + cloudCumulusDepth * 0.3));

            //float powder        = 1.0 - expf(-density * 22.0);
            float powder        = 8.0 * (1.0 - 0.97 * expf(-extinction * 18.0 * powderBias));     //this is loosely based on spectrum
            float anisoPowder   = min(powder, 1.0 - expf(-lightOD * 2.0));
                anisoPowder     = mix(anisoPowder, 1.0, pFade);

            vec3 phaseG         = pow(vec3(0.45, 0.25, 0.95), vec3(1.0 + lightOD));

            for (uint j = 0; j < 8; ++j) {
                float n     = float(j);

                float td    = sigmaT * pow(0.5, n);
                float phase = cloudPhase(vDotL, pow(0.5, n), phaseG);

                stepScatter.x  += expf(-lightOD * td) * phase * td;
                stepScatter.y  += expf(-skyOD * td) * td;
            }

            stepScatter.xy *= vec2(anisoPowder, powder);

            stepScatter.z  += bounceOD * powder;

            stepScatter     = (sunlight * stepScatter.x * sqrt2) + (skylight * stepScatter.y) + (bouncelight * stepScatter.z);
            stepScatter     = mix(skyColor * sigmaT, stepScatter, atmosFade);
            scattering     += stepScatter * (integral * transmittance);

            transmittance  *= stepT;
        }

        transmittance       = linStep(transmittance, cloudTransmittanceThreshold, 1.0);

        totalScattering    += scattering;
        totalTransmittance *= transmittance;
    }
    #endif

    #ifdef cloudCirrusEnabled
    // --- planar cirrus clouds --- //
    bool isBelowPlane   = eyeAltitude < cloudCirrusPlaneY;
    bool visiblePlane   = worldDir.y > 0.0 && isBelowPlane || worldDir.y < 0.0 && !isBelowPlane;

    if (visiblePlane && (!isBelowPlane || totalTransmittance > 1e-10)) {
        vec3 plane      = worldDir * ((cloudCirrusPlaneY - eyeAltitude) * rcp(worldDir.y));

        float airDist   = length(plane);
            //airDist     = mix(min(airDist, cloudVolumeClip), 9e4, within);

        vec3 airmass    = airmass(vec3(0.0, planetRad + eyeAltitude, 0.0), worldDir, airDist, 3) * rcp(60000.0);

        vec3 rPos       = plane + cameraPosition;

        const float sigmaT  = 0.03;

        float dist      = distance(rPos, cameraPosition);

        vec3 scattering = vec3(0.0);
        float transmittance = 1.0;
        const float rLength = cloudCirrusDepth / tau;

        if (dist < cloudCirrusClip) {
            float density   = cloudCirrusShape(rPos);

            if (density > 0.0) {
                float extinction    = density * sigmaT;
                float stepT         = exp(-extinction * rLength);
                float integral      = (1.0 - stepT) * rcp(sigmaT);

                vec3 atmosFade  = expf(-max(airScatterMat * airmass.xy * dist, 0.0));

                if (max3(atmosFade) < 1e-4) {
                    scattering     += (skyColor * sigmaT) * (integral * transmittance);
                    transmittance  *= stepT;
                } else {
                    float lightOD       = cloudCirrusLightOD(rPos, 6, cloudLightDir);
                    float skyOD         = cloudCirrusLightOD(rPos, 4, vec3(0.0, 1.0, 0.0));

                    float powder        = 8.0 * (1.0 - 0.97 * expf(-extinction * 38.0));     //this is loosely based on spectrum
                        powder          = powder * 0.5 + 0.5;
                    float anisoPowder   = mix(powder, 1.0, pFade);

                    vec3 phaseG         = pow(vec3(0.45, 0.25, 0.95), vec3(1.0 + lightOD));

                    for (uint j = 0; j < 8; ++j) {
                        float n     = float(j);

                        float td    = sigmaT * pow(0.5, n);
                        float phase = cloudPhase(vDotL, pow(0.5, n), phaseG);

                        scattering.x   += expf(-lightOD * td) * anisoPowder * phase * td;
                        scattering.y   += expf(-skyOD * td) * powder * td;
                    }

                    scattering      = (sunlight * scattering.x * sqrt2) + (skylight * scattering.y);
                    scattering      = mix(skyColor * sigmaT, scattering, atmosFade);
                    scattering      = scattering * (integral * transmittance);

                    transmittance  *= stepT;
                }
            }
        }

        if (isBelowPlane) {
            totalScattering    += scattering * totalTransmittance;
            totalTransmittance *= transmittance;
        } else {
            totalScattering     = totalScattering * transmittance + scattering;
            totalTransmittance *= transmittance;
        }
    }
    #endif

    return vec4(totalScattering, totalTransmittance);
}

vec4 cloudSystemReflected(vec3 worldDir, float vDotL, float dither, vec3 skyColor) {
    vec3 totalScattering    = vec3(0.0);
    float totalTransmittance = 1.0;

    vec3 sunlight       = (worldTime>23000 || worldTime<12900) ? lightColor[0] : lightColor[2];
    vec3 skylight       = lightColor[1] * rcp(2.0);

    float pFade         = saturate(mieHG(vDotL, 0.65));

    const float eyeAltitude = 64.0;
    vec3 camPos     = vec3(cameraPosition.x, eyeAltitude, cameraPosition.z);
    const uint volumeSamples = 45;

    // --- volumetric clouds --- //
    #ifdef cloudVolumeEnabled
    bool visibleVol = worldDir.y > 0.0;

    float lightNoise = ditherGradNoise();

    if (visibleVol) {

        vec3 bottom     = worldDir * ((cloudCumulusAlt - eyeAltitude) * rcp(worldDir.y));
        vec3 top        = worldDir * ((cloudCumulusMaxY - eyeAltitude) * rcp(worldDir.y));

        float airDist   = length(worldDir.y < 0.0 ? bottom : top);
            airDist     = min(airDist, cloudVolumeClip);

        vec3 airmass    = airmass(vec3(0.0, planetRad + eyeAltitude, 0.0), worldDir, airDist, 2) * rcp(40000.0);

        vec3 start      = bottom;
        vec3 end        = top;

        const float baseLength  = cloudCumulusDepth / float(volumeSamples);
        float stepLength    = length(end - start) * rcp(float(volumeSamples));
        float stepCoeff     = stepLength * rcp(baseLength);
            stepCoeff       = 0.45 + clamp(stepCoeff - 1.1, 0.0, 2.5) * 0.5;

        uint steps          = uint(volumeSamples * stepCoeff);

        vec3 rStep          = (end - start) * rcp(float(steps));
        vec3 rPos           = rStep * dither + start + camPos;
        float rLength       = length(rStep);

        vec3 scattering     = vec3(0.0);
        float transmittance = 1.0;

        vec3 bouncelight    = vec3(0.6, 1.0, 0.8) * sunlight * rcp(pi * 12.0);

        const float sigmaA  = 1.0;
        const float sigmaT  = 0.09;

        for (uint i = 0; i < steps; ++i, rPos += rStep) {
            if (transmittance < cloudTransmittanceThreshold) break;
            if (rPos.y < cloudCumulusAlt || rPos.y > cloudCumulusMaxY) continue;

            float dist  = distance(rPos, camPos);
            if (dist > cloudVolumeClip) continue;

            float powderBias;
            float density = cloudCumulusShape(rPos, powderBias);
            if (density <= 0.0) continue;

            float extinction    = density * sigmaT;
            float stepT         = exp(-extinction * rLength);
            float integral      = (1.0 - stepT) * rcp(sigmaT);

            float airmassMult = 1.0 + sstep(dist, cloudVolumeClip * 0.9, cloudVolumeClip); 
            vec3 atmosFade = expf(-max(airScatterMat * airmass.xy * (dist * airmassMult), 0.0));

            if (max3(atmosFade) < 1e-4) {
                scattering     += (skyColor * sigmaT) * (integral * transmittance);
                transmittance  *= stepT;

                continue;
            }

            vec3 stepScatter    = vec3(0.0);

            float lightOD       = cloudVolumeLightOD(rPos, 4);
            
            #ifdef cloudVolumeSkylightDither
                float skyOD         = cloudVolumeSkyOD(rPos, 3, lightNoise);
            #else
                float skyOD         = cloudVolumeSkyOD(rPos, 3);
            #endif
            
            float bounceOD      = cube(1.0 - linStep(rPos.y, cloudCumulusAlt, cloudCumulusAlt + cloudCumulusDepth * 0.3));

            //float powder        = 1.0 - expf(-density * 22.0);
            float powder        = 8.0 * (1.0 - 0.97 * expf(-extinction * 18.0 * powderBias));     //this is loosely based on spectrum
            float anisoPowder   = min(powder, 1.0 - expf(-lightOD * 2.0));
                anisoPowder     = mix(anisoPowder, 1.0, pFade);

            vec3 phaseG         = pow(vec3(0.45, 0.25, 0.95), vec3(1.0 + lightOD));

            for (uint j = 0; j < 5; ++j) {
                float n     = float(j);

                float td    = sigmaT * pow(0.5, n);
                float phase = cloudPhase(vDotL, pow(0.5, n), phaseG);

                stepScatter.x  += expf(-lightOD * td) * phase * td;
                stepScatter.y  += expf(-skyOD * td) * td;
            }

            stepScatter.xy *= vec2(anisoPowder, powder);

            stepScatter.z  += bounceOD * powder;

            stepScatter     = (sunlight * stepScatter.x * sqrt2) + (skylight * stepScatter.y) + (bouncelight * stepScatter.z);
            stepScatter     = mix(skyColor * sigmaT, stepScatter, atmosFade);
            scattering     += stepScatter * (integral * transmittance);

            transmittance  *= stepT;
        }

        transmittance       = linStep(transmittance, cloudTransmittanceThreshold, 1.0);

        totalScattering    += scattering;
        totalTransmittance *= transmittance;
    }
    #endif

    #ifdef cloudCirrusEnabled
    // --- planar cirrus clouds --- //
    bool visiblePlane   = worldDir.y > 0.0;

    if (visiblePlane && totalTransmittance > 1e-10) {
        vec3 plane      = worldDir * ((cloudCirrusPlaneY - eyeAltitude) * rcp(worldDir.y));

        float airDist   = length(plane);
            //airDist     = mix(min(airDist, cloudVolumeClip), 9e4, within);

        vec3 airmass    = airmass(vec3(0.0, planetRad + eyeAltitude, 0.0), worldDir, airDist, 3) * rcp(60000.0);

        vec3 rPos       = plane + camPos;

        const float sigmaT  = 0.03;

        float dist      = distance(rPos, camPos);

        vec3 scattering = vec3(0.0);
        float transmittance = 1.0;
        const float rLength = cloudCirrusDepth / tau;

        if (dist < cloudCirrusClip) {
            float density   = cloudCirrusShape(rPos);

            if (density > 0.0) {
                float extinction    = density * sigmaT;
                float stepT         = exp(-extinction * rLength);
                float integral      = (1.0 - stepT) * rcp(sigmaT);

                vec3 atmosFade  = expf(-max(airScatterMat * airmass.xy * dist, 0.0));

                if (max3(atmosFade) < 1e-4) {
                    scattering     += (skyColor * sigmaT) * (integral * transmittance);
                    transmittance  *= stepT;
                } else {
                    float lightOD       = cloudCirrusLightOD(rPos, 4, cloudLightDir);
                    float skyOD         = cloudCirrusLightOD(rPos, 3, vec3(0.0, 1.0, 0.0));

                    float powder        = 8.0 * (1.0 - 0.97 * expf(-extinction * 38.0));     //this is loosely based on spectrum
                        powder          = powder * 0.5 + 0.5;
                    float anisoPowder   = mix(powder, 1.0, pFade);

                    vec3 phaseG         = pow(vec3(0.45, 0.25, 0.95), vec3(1.0 + lightOD));

                    for (uint j = 0; j < 5; ++j) {
                        float n     = float(j);

                        float td    = sigmaT * pow(0.5, n);
                        float phase = cloudPhase(vDotL, pow(0.5, n), phaseG);

                        scattering.x   += expf(-lightOD * td) * anisoPowder * phase * td;
                        scattering.y   += expf(-skyOD * td) * powder * td;
                    }

                    scattering      = (sunlight * scattering.x * sqrt2) + (skylight * scattering.y);
                    scattering      = mix(skyColor * sigmaT, scattering, atmosFade);
                    scattering      = scattering * (integral * transmittance);

                    transmittance  *= stepT;
                }
            }
        }

        totalScattering    += scattering * totalTransmittance;
        totalTransmittance *= transmittance;
    }
    #endif

    return vec4(totalScattering, totalTransmittance);
}

#ifdef lightHqFilterEnabled
 /* - */
#endif

/* ------ sky reflection ------ */
vec2 sincos(float x) {
    return vec2(sin(x), cos(x));
}

vec3 projectSphere(vec2 coord) {
    coord  *= vec2(tau, pi);
    vec2 lon = sincos(coord.x) * sin(coord.y);
    return vec3(lon.x, cos(coord.y), lon.y);
}

void main() {
    #if (defined lightHqFilterEnabled && defined lightmapSmoothingEnabled)
    lightmapData    = clamp16F(lightmap3x3(colortex3, depthtex0, colortex6, coord, ATROUS_SIZE));
    #endif
    skyboxData      = texture(colortex5, coord);

    #if (defined cloudVolumeEnabled || defined cloudCirrusEnabled)
    const float clod = sqrt(CLOUD_RENDER_LOD);

    vec2 cloudCoord = (coord - vec2(1.0 - rcp(clod), 0.0)) * clod;

    if (clamp(cloudCoord, -pixelSize * clod, 1.0 + pixelSize * clod) == cloudCoord) {
        cloudCoord  = saturate(cloudCoord);

        float depth = depthMax3x3(depthtex0, cloudCoord, pixelSize * clod);

        if (!landMask(depth)) {
            vec3 viewPos    = screenToViewSpace(vec3(cloudCoord, 1.0));
            vec3 viewDir    = normalize(viewPos);
            vec3 scenePos   = viewToSceneSpace(viewPos);
            vec3 worldDir   = normalize(scenePos);

            vec3 skyColor   = textureBicubic(colortex5, projectSky(worldDir)).rgb;

            skyboxData      = cloudSystem(worldDir, dot(viewDir, cloudLightDirView), ditherBluenoise(), skyColor);
        } else {
            skyboxData.a    = 1.0;
        }
    }
    #endif

    vec2 reflectionCoord    = (coord - vec2(0.0, exp2(-SKY_RENDER_LOD) * 2.0)) * skyCaptureResolution;

    if (isOnScreenDownscale(reflectionCoord, pixelSize, 4)) {
        vec2 coord          = saturate(reflectionCoord);
        vec3 worldDir       = projectSphere(coord) * vec3(1.0, 1.0, -1.0);

        vec3 skyColor       = texture(colortex5, projectSky(worldDir)).rgb;

        vec4 cloudData      = cloudSystemReflected(worldDir, dot(mat3(gbufferModelView) * worldDir, cloudLightDirView), ditherBluenoiseStatic(), skyColor);

        skyColor    = skyColor * cloudData.a + cloudData.rgb;

        skyboxData  = vec4(skyColor, 1.0);
    }

    skyboxData  = clamp16F(skyboxData);
}