#version 400 compatibility

/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#include "/lib/head.glsl"
#include "/lib/util/encoders.glsl"
#include "/lib/util/colorspace.glsl"

const int shadowMapResolution   = 2048;     //[512 1024 1536 2048 2560 3072 3584 4096 6144 8192 16384]
const float shadowDistance      = 128.0;

const bool shadowHardwareFiltering = false;

in vec2 coord;

flat in mat4x3 lightColor;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;

uniform sampler2D noisetex;

uniform sampler2D depthtex0;

uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D shadowcolor0;

uniform int frameCounter;

uniform float aspectRatio;
uniform float far, near;
uniform float lightFlip;
uniform float sunAngle;
uniform float viewHeight, viewWidth;

uniform vec2 taaOffset;
uniform vec2 viewSize, pixelSize;

uniform vec3 lightvec, lightvecView;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;
uniform mat4 shadowModelView, shadowModelViewInverse;
uniform mat4 shadowProjection, shadowProjectionInverse;


/* ------ includes ------ */
#include "/lib/util/transforms.glsl"

#define FUTIL_LINDEPTH
#define FUTIL_MAT16
#define FUTIL_LIGHTMAP
#include "/lib/fUtil.glsl"

#include "/lib/light/diffuse.glsl"

#include "/lib/light/warp.glsl"

#include "/lib/frag/bluenoise.glsl"
#include "/lib/frag/dither.glsl"

#define SOLIDPASS

#include "/lib/light/shadow.glsl"

#include "/lib/atmos/phase.glsl"

#include "/lib/light/subsurface.glsl"

vec4 textureBilateral(sampler2D tex, sampler2D depth, const int lod, float fdepth) {
    vec4 data   = vec4(0.0);
    float sum   = 0.0;
    ivec2 posD  = ivec2(coord*viewSize);
    ivec2 posT  = ivec2(coord*viewSize*rcp(float(lod)));
    vec3 zmult  = vec3((far*near)*2.0, far+near, far-near);
        fdepth  = depthLinear(fdepth);
    
    for (int i = -1; i<2; i++) {
        for (int j = -1; j<2; j++) {
            ivec2 tcDepth = posD + ivec2(i, j)*lod;
            float dsample = depthLinear(texelFetch(depth, tcDepth, 0).x);
            float w     = abs(dsample-fdepth)*zmult.x<1.0 ? 1.0 : 1e-5;
            ivec2 ct    = posT + ivec2(i, j);
            data       += texelFetch(tex, ct, 0)*w;
            sum        += w;
        }
    }
    data *= rcp(sum);

    return data;
}

//fuck that shit
/*
float getContactShadow(sampler2D depth, vec3 viewPos, float dither, float sceneDepth, float nDotV) {
    const uint steps = 16;
    const float maxDistance = 0.01;

    vec3 screenPos  = viewToScreenSpace(viewPos);
	//float rLength   = ((viewPos.z + lightvecView.z * far*sqrt(3.)) > -near) ? (-near -viewPos.z) / lightvecView.z : far*sqrt(3.);
    float rLength   = maxDistance * rcp(float(steps)) * (-near - viewPos.z) * rcp(abs(lightvecView.z));
        //rLength     = max(rLength, 1e-4);
    vec3 lDir       = viewToScreenSpace(viewPos + lightvecView * rLength) - screenPos;
        //lDir       *= rcp(max(abs(lDir) * rcp(pixelSize.x), abs(lDir.y) * rcp(pixelSize.y)));

    float minDelta  = mix(1e-4, 1e-1, pow5(saturate(nDotV * 0.5 + 0.5)));
    
    vec3 rStep      = lDir * pi;
    vec3 rPos       = screenPos + rStep * dither;
        rPos.xy    += taaOffset * 0.5;

    for (uint i = 0; i < steps; ++i, rPos += rStep) {

        if (saturate(rPos.xy) != rPos.xy) continue;

        float currDepth     = texture(depth, rPos.xy).x;
        if (currDepth == 1.0) continue;

        if (currDepth < rPos.z) {
            //return depthLinear(currDepth) * 64;
            vec3 currViewPos    = screenToViewSpace(vec3(rPos.xy, currDepth));
            float weight        = dot(normalize(currViewPos - viewPos), lightvecView);
            if (weight < 0.5) continue;

            float linZ          = depthLinear(rPos.z);
            float delta         = abs(depthLinear(currDepth) - linZ) * rcp(linZ);

            //if (delta < minDelta) continue;

            if (delta < maxDistance) return expf(viewPos.z * 1.5);
        }

        //if (delta > 0.001 && delta < maxDistance) return 0.0;
    }
    //return depthLinear(sceneDepth) * 64;
    return 1.0;
}*/

/*
Screenspace Contact shadows based on Spectrum by Zombye.
Because I have a hard time with screenspace RT effects for some reason...
*/

float viewToScreenSpace(float depth) {
	return (gbufferProjection[2].z * depth + gbufferProjection[3].z) * 0.5 * rcp(-depth) + 0.5;
}

float AscribeDepth(float depth, float ascribeAmount) {  //from Spectrum, didn't feel like redoing that function for no reason
	depth = 1.0 - 2.0 * depth;
	depth = (depth + gbufferProjection[2].z * ascribeAmount) / (1.0 + ascribeAmount);
	return 0.5 - 0.5 * depth;
}
float GetLinearDepth(sampler2D depthSampler, vec2 coord) {
    // Interpolates after linearizing, significantly reduces a lot of issues for screen-space shadows
    coord = coord * viewSize + 0.5;

    vec2  f = fract(coord);
    ivec2 i = ivec2(coord - f);

    vec4 s = textureGather(depthSampler, i / viewSize) * 2.0 - 1.0;
    s = 1.0 / (gbufferProjectionInverse[2].w * s + gbufferProjectionInverse[3].w);

    s.xy = mix(s.wx, s.zy, f.x);
    return mix(s.x,  s.y,  f.y) * gbufferProjectionInverse[3].z;
}
float getContactShadow(sampler2D depth, vec3 viewPos, float dither, float sceneDepth, float nDotV) {
    const uint steps = 16;
    const uint stride = 4;
    
    vec3 screenPos  = viewToScreenSpace(viewPos, true);

    vec3 rStep  = viewPos + abs(viewPos.z) * lightvecView;
        rStep   = viewToScreenSpace(rStep, true) - screenPos;
        rStep  *= min3((step(0.0, rStep) - screenPos) / rStep);

        screenPos.xy *= viewSize;
        rStep.xy   *= viewSize;
        //screenPos.xy -= taaOffset * 0.5;

        rStep  *= rcp(abs(abs(rStep.x) < abs(rStep.y) ? rStep.y : rStep.x));

    vec2 stepsToEnd     = (step(0.0, rStep.xy) * viewSize - screenPos.xy) / rStep.xy;
    uint maxIterations  = min(uint(ceil(min(min(stepsToEnd.x, stepsToEnd.y), max2(viewSize)) * rcp(float(stride)))), steps);

    vec3 startPos   = screenPos;

    bool hit    = false;

    float noise     = floor(stride * dither + 1.0);

    for (uint i = 0; i < maxIterations && !hit; ++i) {
        float pixelSteps    = float(i * stride) + noise;
        screenPos   = startPos + pixelSteps * rStep;

        float maxZ  = screenPos.z;
        float minZ  = rStep.z > 0.0 && i == 0 ? startPos.z : screenPos.z - float(stride) * abs(rStep.z);

        if (1.0 < minZ || maxZ < 0.0) break;

        float d     = texelFetch(depthtex0, ivec2(screenPos.xy), 0).x;
        float ascribedD = AscribeDepth(d, 1e-2 * (i == 0 ? noise : float(stride)) * gbufferProjectionInverse[1].y);
        float dInterp = viewToScreenSpace(GetLinearDepth(depthtex0, screenPos.xy * pixelSize));
        float ascribedDInterp = AscribeDepth(dInterp, 1e-2 * (i == 0 ? noise : float(stride)) * gbufferProjectionInverse[1].y);

        hit = maxZ >= d && minZ <= ascribedD && maxZ >= dInterp && minZ <= ascribedDInterp && d > 0.65 && d < 1.0;
    }

    return float(!hit);
}

#include "/lib/light/brdf.glsl"

float bayer2e(vec2 a){
    a = floor(a);
    return fract( dot(a, vec2(.5, a.y * .75)) );
}
#define bayer4e(a)   (bayer2e( .5*(a))*.25+bayer2e(a))

#define m vec3(31,63,31)
float encode3x8(vec3 a){
    float dither = bayer4e(gl_FragCoord.xy);
    a += (dither-.5) / m;
    a = saturate(a);
    ivec3 b = ivec3(a*m);
    return float( b.r|(b.g<<5)|(b.b<<11) ) / 65535.;
}
#undef m

void main() {
    vec4 sceneColor   = stex(colortex0);
    float sceneDepth = stex(depthtex0).x;

    vec4 tex2       = stex(colortex2);
        tex2.y      = encode3x8(sqrt(saturate(sceneColor.rgb)));

    #if DEBUG_VIEW!=3
    if (landMask(sceneDepth)) {
        vec4 tex1       = stex(colortex1);
        vec2 sceneLmap  = decode2x8(tex1.z);
        vec3 sceneNormal = decodeNormal(tex1.xy);
        vec3 viewNormal = mat3(gbufferModelView) * sceneNormal;

        int matID       = decodeMatID16(tex2.x);

        //#if DEBUG_VIEW==1
        //sceneColor.rgb    = vec3(0.25);
        //#endif

        vec3 viewPos    = screenToViewSpace(vec3(coord, sceneDepth));
        vec3 viewDir    = normalize(viewPos);
        vec3 scenePos   = viewToSceneSpace(viewPos);

        #ifdef lightmapSmoothingEnabled
        vec3 tex3       = stex(colortex3).xyz;
            sceneLmap   = tex3.xy;
            sceneColor.a = tex3.z;
        #endif

        float ao        = sqr(sceneColor.a)*0.66+0.34;
        
        #ifdef ambientOcclusionEnabled
            vec4 upscale3 = textureBilateral(colortex3, depthtex0, 2, sceneDepth);
                ao     *= upscale3.w;
        #endif
        
        /* ------ lighting calculation ------ */
        //~6 fps
        
        vec2 lmap       = sceneLmap;
        lmap.y          = cube(lmap.y);

        #if diffuseModel == 0
        float diff      = diffuseOrenNayar(viewNormal, -viewDir, lightvecView, 1.0);
        #else
        float diff      = diffuseLambert(sceneNormal);
        #endif

        #ifndef subsurfaceScatteringEnabled
            if (matID == 2) diff = diff*0.3+0.7;
            if (matID == 4) diff = sqrt(diff);
        #endif

        shadowData shadow = shadowData(1.0, vec3(1.0), vec3(0.0));

        #ifdef subsurfaceScatteringEnabled
            if (matID == 2 || matID == 4) shadow = getShadowSubsurface(diff>0.0, scenePos, normalize(viewPos), sceneColor.rgb);
            else shadow = getShadowRegular(diff>0.0, scenePos);    //~3fps, almost for free
        #else
            shadow = getShadowRegular(diff>0.0, scenePos);    //~3fps, almost for free
        #endif

        float diffLit       = min(diff, shadow.shadow);

        #ifdef contactShadowsEnabled
            if (diffLit > 0.0) diffLit *= getContactShadow(depthtex0, viewPos, ditherBluenoise(), sceneDepth, dot(viewNormal, viewDir));
        #endif

        vec3 directCol      = (sunAngle<0.5 ? lightColor[0] : lightColor[2]) * lightFlip;
        vec3 directLight    = diffLit * shadow.color * directCol;

        materialProperties material     = materialProperties(1.0, 0.04, false, mat2x3(1.0));

        #ifdef subsurfaceScatteringEnabled
            directLight    += shadow.subsurfaceScatter * directCol;
        #endif

        vec3 indirectLight  = sqr(lmap.y)*lightColor[1];
            indirectLight  += lmap.y * directCol * 0.06;
            indirectLight  += vec3(0.5, 0.7, 1.0) * (0.01 * minLightLuma);
            indirectLight  *= ao;

        vec3 result     = directLight + indirectLight;
            result     += getBlocklightMap(lightColor[3], lmap.x)*ao;

        #ifdef emitterBoostEnabled
        if (matID == 5) {
            vec3 emitterCol     = mix(lightColor[2], vec3(v3avg(lightColor[3])), emitterRecoloring);
                emitterCol     *= cubeSmooth(saturate(v3avg(sceneColor.rgb) * 1.5));
            result += emitterCol * pi;
        }
        #endif

        sceneColor.rgb *= result;

        #if DEBUG_VIEW == 1
            sceneColor.rgb = 0.5 * result;
        #endif

        #ifdef specularBRDFEnabled
        if (diffLit > 0.0) {
            vec3 brdf   = BRDF(-viewDir, lightvecView, viewNormal, material) * directCol;
            //vec3 fresnel = BRDFfresnel(-viewDir, viewNormal, material);
            sceneColor.rgb = sceneColor.rgb + brdf;
            //sceneColor.rgb *= fresnel;
        }
        #endif

        #ifdef deffNAN
        if (isnan3(sceneColor.rgb) || isinf3(sceneColor.rgb)) sceneColor.g = 100.0;
        #endif

        //sceneColor.rgb    = shadow.subsurfaceScatter;
        //sceneColor.rgb  = vec3(getContactShadow(depthtex0, viewPos, ditherBluenoise(), sceneDepth, dot(viewNormal, viewDir)));

        #if DEBUG_VIEW==2
        sceneColor.rgb    = ao * (sunAngle<0.5 ? lightColor[0] : lightColor[3]);
        #endif
    }
    #endif

    /*DRAWBUFFERS:023*/
    gl_FragData[0]  = makeDrawbuffer(sceneColor);
    gl_FragData[1]  = clampDrawbuffer(tex2);
    gl_FragData[2]  = vec4(0.0);
}