#version 400 compatibility

/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#include "/lib/head.glsl"
#include "/lib/util/encoders.glsl"

in vec2 coord;

flat in mat3x3 lightColor;

uniform sampler2D colortex0;
uniform sampler2D colortex2;
uniform sampler2D colortex3;

uniform sampler2D depthtex0;
uniform sampler2D depthtex1;

uniform int isEyeInWater;

uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;

uniform vec2 taaOffset;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;


/* ------ includes ------ */
#define FUTIL_TBLEND
#define FUTIL_MAT16
#include "/lib/fUtil.glsl"

#include "/lib/util/transforms.glsl"


vec3 simple_fog(vec3 scenecolor, float d, vec3 color) {
    float dist      = max(0.0, d-32.0);
    float density   = 1.0-exp(-dist*1e-3);

    return mix(scenecolor, vec3(0.0), density);
}
vec3 water_fog(vec3 scenecolor, float d, vec3 color) {
    float dist      = max(0.0, d);
    float density   = dist*6.5e-1;
    vec3 scatter   = 1.0-exp(-min(density*0.8, 64e-1)*vec3(0.02, 0.24, 1.0));
    vec3 transmittance = exp(-density*vec3(1.0, 0.28, 0.06)*1.2);

    return scenecolor*transmittance + color*scatter*0.3;
}

void main() {
    vec4 sceneColor   = stex(colortex0);  //that macro certainly makes it neater
    //vec4 tex1       = stex(colortex1);
    vec4 tex2       = stex(colortex2);
    vec4 tex3       = stex(colortex3);

    int mat_id      = decodeMatID16(tex2.x);

    float scenedepth0 = stex(depthtex0).x;
    vec3 viewpos0     = screenToViewSpace(vec3(coord, scenedepth0));
    vec3 spos0        = viewToSceneSpace(viewpos0);

    float scenedepth1 = stex(depthtex1).x;
    vec3 viewpos1     = screenToViewSpace(vec3(coord, scenedepth1));
    vec3 spos1        = viewToSceneSpace(viewpos1);

    bool translucent = (scenedepth0<scenedepth1);

    bool water      = mat_id == 102;

    vec3 translucent_albedo = sqr(decode3x8(tex2.g));
    
    float dist0     = distance(spos0, gbufferModelViewInverse[3].xyz);
    float dist1     = distance(spos1, gbufferModelViewInverse[3].xyz);

    if (translucent && isEyeInWater==0){
        if (water) sceneColor.rgb = water_fog(sceneColor.rgb, dist1-dist0, lightColor[1]);
        else if (landMask(scenedepth1)) sceneColor.rgb = simple_fog(sceneColor.rgb, dist1-dist0, lightColor[1]);
    }

    if (landMask(scenedepth1) && isEyeInWater==1 && translucent) sceneColor.rgb = simple_fog(sceneColor.rgb, dist1-dist0, lightColor[1]);

    sceneColor.rgb    = blendTranslucencies(sceneColor.rgb, tex3, translucent_albedo);

    if (landMask(scenedepth0) && isEyeInWater==0) sceneColor.rgb = simple_fog(sceneColor.rgb, dist0, lightColor[1]);

    if (isEyeInWater==1) sceneColor.rgb = water_fog(sceneColor.rgb, dist0, lightColor[1]);

    if (mat_id == 3) {
        if (landMask(scenedepth0)) sceneColor.rgb = sceneColor.rgb*0.6 + vec3(0.5)*v3avg(sceneColor.rgb);
        else sceneColor.rgb = sceneColor.rgb*0.7 + vec3(0.8)*v3avg(sceneColor.rgb);
    }

    /*DRAWBUFFERS:0*/
    gl_FragData[0]  = makeDrawbuffer(sceneColor);
}