#version 400 compatibility

/*
====================================================================================================

    Copyright (C) 2020 RRe36

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://rre36.github.io/license/

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#include "/lib/head.glsl"
#include "/lib/util/encoders.glsl"

/*
stand in for sky pass
*/

const int noiseTextureResolution = 256;

in vec2 coord;

flat in mat3x3 lightColor;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D depthtex1;

uniform sampler2D noisetex;

uniform float aspectRatio;
uniform float far, near;

uniform vec2 taaOffset;

uniform vec3 upvec, upvecView;
uniform vec3 sunvec, sunvecView;
uniform vec3 moonvec, moonvecView;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;

/* ------ includes ------ */
#define FUTIL_LINDEPTH
#include "/lib/fUtil.glsl"

#include "/lib/util/transforms.glsl"

/* ------ ambient occlusion and gi ------ */

#include "/lib/light/ao.glsl"

#define noise_2d_only
#include "/lib/frag/noise.glsl"

vec3 getStars(vec3 spos, vec3 svec) {
    vec3 plane  = svec/(svec.y+length(svec.xz)*0.66);
        if (svec.y < 0.0) plane  = (-svec)/(-svec.y+length(svec.xz)*0.66);
    //float rot   = worldTime*rcp(2400.0);
    //plane.x    += rot*0.6;
    //plane.yz    = rotate_coord(plane.yz, (25.0/180.0)*pi);
    vec2 uv1    = floor((plane.xz)*768)/768;
    vec2 uv2    = (plane.xz)*0.04;

    vec3 starcol = vec3(0.5, 0.68, 1.0);
        starcol  = mix(starcol, vec3(1.0, 0.7, 0.6), noise_2d(uv2).x);
        starcol  = normalize(starcol)*(noise_2d(uv2*1.5).x+1.0);

    float star  = 1.0;
        star   *= noise_2d(uv1).x;
        star   *= noise_2d(uv1+0.1).x;
        star   *= noise_2d(uv1+0.26).x;

    star        = max(star-0.25, 0.0);
    star        = saturate(star*4.0);

    return star*starcol*2.0;
}

void main() {
    vec4 sceneColor   = stex(colortex0);
    float sceneDepth = stex(depthtex1).x;

    vec3 viewpos    = screenToViewSpace(vec3(coord, sceneDepth));
    vec3 viewvec    = normalize(viewpos);
    vec3 scenepos   = viewToSceneSpace(viewpos);
    vec3 svec       = normalize(scenepos);

    if (!landMask(sceneDepth)) {
        sceneColor.rgb = getStars(scenepos, svec);
    }

    vec4 return3    = vec4(0.0);
    vec3 return6    = vec3(0.0);

    #ifdef lightmapSmoothingEnabled
    if (landMask(sceneDepth)) {
        vec4 tex1   = stex(colortex1);
        return3.xyz = vec3(decode2x8(tex1.z), sceneColor.a);
        return6     = decodeNormal(tex1.xy) * 0.5 + 0.5;
    }
    #endif

    #ifdef ambientOcclusionEnabled
        vec2 ao_coord   = (coord)*2.0;

        if (clamp(ao_coord, -0.003, 1.003) == ao_coord) {
            vec2 coord  = ao_coord;
            float d     = depthMax3x3(depthtex1, ao_coord, pixelSize*sqrt(2.0));

            if (landMask(d)) {
                float sceneDepth = stex(depthtex1).x;
                vec4 tex1       = stex(colortex1);

                vec3 viewpos    = screenToViewSpace(vec3(coord, sceneDepth));

                vec3 sceneNormal = decodeNormal(tex1.xy);
                vec3 viewnormal = normalize(mat3(gbufferModelView) * sceneNormal);

                float ao    = calculate_dbao(depthtex1, sceneDepth, coord, ditherBluenoise());
                //float ao    = calculate_ssao(depthtex1, viewpos, viewnormal, ditherBluenoise());

                return3.w   = ao;
            } else {
                return3.w   = 1.0;
            }
        }
    #endif

    /*DRAWBUFFERS:036*/
    gl_FragData[0]  = makeDrawbuffer(sceneColor);
    gl_FragData[1]  = clampDrawbuffer(return3);
    gl_FragData[2]  = clampDrawbuffer(return6);
}